#! /use/bin/perl -w
# Created by Yiming Cheng on 2005/12/17
# Function: generate the score matrix for 15 elements of the sequence file.
# Comments: modified by Yiming on 2006/05/04
# 06/13/07 bug fix my Mike Tsai, print to score file prints incorrect number of columns

use strict;
use Getopt::Std;

sub exit_help{
	print "\nUsage: perl match_seq.pl -i input.seq -o output.res [-n 1|-1] [-l location]\n";
	print "Options: \n";
	print "-i input --- the input file containing FASTA format sequences file.\n";
	print "-o output --- the output file for SVM prediction.\n";
	print "[-n 1|-1] --- the index of the data, 1 means positive data and -1 means negative data, default 1 Just for the format.\n";
	print "[-l location] --- it will do one hit based on the given location,if 0, do multi hit, default 0.\n";
	print "	      --- if the sequence lenght is less than 200,(need to be greater than 120),this option is omitted.\n\n";
	exit;
}

my %args;
getopt("ionl",\%args); 
my ($input,$output,$index,$location);
&ini_options();

my $mat_dir ="./data/pssm/";
my @file_ext = ("AUE1","AUE2","AUE3","AUE4","CUE1","CUE2","CDE1","CDE2","CDE3","CDE4","ADE1","ADE2","ADE3","ADE4","ADE5");
my @reg_start = (0,0,0,0,60,60,100,100,100,100,140,140,140,140,140);
my @reg_end = (60,60,60,60,100,100,140,140,140,140,200,200,200,200,200);

my $num_ext = @file_ext;
my (%mat,%mat_size,%score_mat,%mat_perc);

my $cnt_seq=0;  
my $line_len = 80;
my $num_file = $input.".num";
open(NUM_FILE,">$num_file");
my $score_file = $input.".score";
open(SCORE,">$score_file");

&load_percentile();
&load_mat();
&scan_seq();	

close(NUM_FILE);
close(SCORE);

sub ini_options{
	$input=$args{i};  #input sequence file
	if (!defined($input)){
		&exit_help();
	}
	$output = $args{o};
	if (!defined($output)) {
		&exit_help();
	}
	$index = $args{n};
	if (!defined($index)){
		$index=1;
	}
	$location = $args{l};
	if (!defined($location)){
		$location=0;
	}
}

sub load_mat{
	my $mat_file;
	for (my $j=0;$j<$num_ext;$j++){
		$mat_file = $mat_dir.$file_ext[$j].".MAT";
		open (MAT,"<$mat_file") or die "Matrix file not found";
		my @nts;
	
		while (<MAT>){
			chomp;
			$_ =~ s/\://g;    # delete ":" in the line
			if ($_=~/Pos/i){
				$_=~s/U/T/g;    
				@nts = split /\t/, $_;  # @nts = "pos A C G T"
				shift @nts;       # @nts = "A C G T"
			}elsif ($_=~/^\d/){
				my @vals=split /\t/, $_;
				my $pos=shift @vals;
				for (my $i=0; $i<@vals;$i++){
					next if($vals[$i] =~/Inf/i);
					$mat{$file_ext[$j]}{$pos}{$nts[$i]}=$vals[$i];
				}
				$mat_size{$file_ext[$j]}++;
			}
		}
		close MAT;
	}
}

sub load_percentile{
	open(PERC,"<./data/percentile");
	while(<PERC>){
		chomp;
		my @line =split(/\t/,$_);
		$mat_perc{$line[0]}{'25'}=$line[3];
		$mat_perc{$line[0]}{'50'}=$line[2];
		$mat_perc{$line[0]}{'75'}=$line[1];
	}
	close(PERC);
}

sub scan_seq{
	open (IN,"<$input") or die "Input file not found";
	open (OUT,">$output");
	
	my ($id,$seq,$anno);
	while (<IN>){
		chomp;
		if ($_=~/^>/){
			$cnt_seq++;
			if ($cnt_seq >1){
				&match_mat($id,$seq);
			}
			($id,$anno)=split /\t/, $_;  #for example, id = ">p.2987.1"
                                                     #anno = "LLID:2987|CONTIG:NT_004559|ST:1|LOC:4512837|CLEVC:6|ACCC:103"
			if ($id=~/^>lcl\|([\w\.]+)/){    # lcl
				$id=$1;
			}elsif($id=~/^>([\w\. ]+)/){    
				$id=$1;
			}else{
				die("problem");
			}
			$seq='';
		}else{
			$seq .= $_;
		}
	}
	&match_mat($id,$seq);
	close(IN);
  close(OUT);
}

#############
sub match_mat{
	my $temp_id=$_[0];   	#id ="p.2987.1";
	my $seq=$_[1];  	#seq ="ATGCATGCCCCCCCCCAGGGTTC..."
	$seq =~ s/ //g;   #delete empty char in the sequence
	my @seq_nts;
  
  	$seq=uc($seq);
  	@seq_nts=split //, $seq;
	my $seq_len= @seq_nts;
	foreach my $mat_name (@file_ext){
		@{$score_mat{$mat_name}}=();
	}
	&getscore($seq);
	############## Start printing score;
	my %temp_score_print=();
	for (my $i =0;$i<$num_ext;$i++){
		my $temp_name = $file_ext[$i];
		my @temp_arr_score = @{$score_mat{$temp_name}};
		my @new_arr_score;
		for (my $j=0;$j<=$#temp_arr_score;$j++){
			if ($temp_arr_score[$j]>$mat_perc{$temp_name}{'75'}){
				push @new_arr_score,"+";
			} elsif ($temp_arr_score[$j]>$mat_perc{$temp_name}{'50'}){
				push @new_arr_score,"|";
			} elsif ($temp_arr_score[$j]>$mat_perc{$temp_name}{'25'}){
				push @new_arr_score,":";
			} elsif ($temp_arr_score[$j]>0){
				push @new_arr_score,".";
			} else {
				push @new_arr_score,"-";
			}
		}
		for (my $j=($#new_arr_score+1);$j<$seq_len;$j++){
			push @new_arr_score,"-";
		}
		push @{$temp_score_print{$temp_name}},@new_arr_score;
	}
        my ($start,$end);
	my $num = int($seq_len/$line_len);
	print SCORE ">$temp_id\n";
        #for (my $i=0; $i<($num+1); $i++)
        for (my $i=0; $i<($num); $i++){
		$start=$line_len*$i;
		if ($start+$line_len>$seq_len){
			$end=($seq_len-1);
		}else{
			$end=$start+$line_len-1;
			#$end=$start+$line_len;
		}
		print SCORE "\n    \t",(join "",@seq_nts[$start..$end]);
		for (my $j=0;$j<$num_ext;$j++){
			print SCORE "\n",$file_ext[$j],"\t",(join "",@{$temp_score_print{$file_ext[$j]}}[$start..$end]);
		}
	}
	print SCORE "\n\n";
	################### End printing score
	
	################# Start generate matrix for svm
	my %mat_max_score=();
	my ($start_site,$end_site);
	if ($seq_len<120){
		print NUM_FILE $temp_id,"\t",0,"\n";
		return(0);
	} elsif ($seq_len<=200){
		print OUT $index,"\t";
		my ($start_site,$end_site);
		for (my $i=0;$i<$num_ext;$i++){
			my @one_score = @{$score_mat{$file_ext[$i]}};
			my $mid_num = int(scalar(@one_score)/2);
			if ($file_ext[$i]=~/AUE/){
				$start_site = 0;
				$end_site = 60-$mat_size{$file_ext[$i]};
			} elsif ($file_ext[$i]=~/CUE/){
				$start_site = $mid_num-40;
				$end_site = $mid_num-$mat_size{$file_ext[$i]};
			} elsif ($file_ext[$i]=~/CDE/){
				$start_site = $mid_num;
				$end_site = $mid_num+40-$mat_size{$file_ext[$i]};
			} elsif ($file_ext[$i]=~/ADE/){
				$start_site = $seq_len-60;
				$end_site = $seq_len-$mat_size{$file_ext[$i]};
			} else {
				die "Not reachable\n";
			}
			print OUT &get_max(join(":",@one_score[$start_site..$end_site])),"\t";
		}
		print OUT "\n";
		print NUM_FILE $temp_id,"\t",1,"\t",0,"\n";
		return(0);
	}
	### sequence length will be greater than 200 following
	if ($location == 0){
		for (my $i=0;$i<$num_ext;$i++){
			my @one_score = @{$score_mat{$file_ext[$i]}};
			$start_site = $reg_start[$i];
			$end_site = ($reg_end[$i]-$mat_size{$file_ext[$i]});
			my @first_arr = @one_score[$start_site..$end_site];
			my $curr_max = &get_max(join(":",@first_arr));
			push @{$mat_max_score{0}},$curr_max;
			my $first_ele;
			for (my $j=1;$j<($seq_len-199);$j++){
				$first_ele = shift @first_arr;
				push @first_arr,$one_score[$end_site];
				$start_site++;
				$end_site++;
				if ($curr_max<=$first_ele){
					$curr_max = &get_max(join(":",@one_score[$start_site..$end_site]));
					push @{$mat_max_score{$j}},$curr_max;
				} else {
					if ($one_score[$end_site]>$curr_max){
						$curr_max = $one_score[$end_site];
						push @{$mat_max_score{$j}},$curr_max;
					} else {
						push @{$mat_max_score{$j}},$curr_max;
					}
				}
			}
		}
		for (my $i=0;$i<($seq_len-199);$i++){
			print OUT $index,"\t",join("\t",@{$mat_max_score{$i}}),"\n";
		}
		print NUM_FILE $temp_id,"\t",($seq_len-199),"\n";
	} else {
		if (($location <100)||($location >($seq_len-100))){
			die "The location you specified is beyond the range of the sequence $temp_id.\n";
		}
		my $i=$location-100;
		print OUT "$index\t";
		for (my $j=0;$j<$num_ext;$j++){
			my @one_score = @{$score_mat{$file_ext[$j]}};
			$start_site = $i+$reg_start[$j];
			$end_site = $i+$reg_end[$j]-$mat_size{$file_ext[$j]};
			print OUT &get_max(join(":",@one_score[$start_site..$end_site])),"\t";

		} 
		print OUT "\n";
		print NUM_FILE $temp_id,"\t",1,"\t",$location,"\n";
	}
}

sub get_max{
	my @temp_score = split(/:/,$_[0]);
	my $max_score = -99;
	for (my $i=0;$i<=$#temp_score;$i++){
		if ($temp_score[$i]>$max_score){
			$max_score = $temp_score[$i];
		}
	}
	if ($max_score == -99){
		die "Never reach!.\n";
	}
	return($max_score);
}

sub getscore{
	my $sequence = $_[0];
	my @sequence_nts = split //,$sequence;
	my $seq_length = scalar(@sequence_nts);
	my $score = 0;
	for (my $i=0;$i<$num_ext;$i++){
		my $mat_name = $file_ext[$i];
		for (my $j=0;$j<=($seq_length-$mat_size{$mat_name});$j++){
			$score=0;
			for (my $k=$j; $k<($j+$mat_size{$mat_name}); $k++){
				if (!exists($mat{$mat_name}{($k-$j+1)}{$sequence_nts[$k]})){
					$score=-15;
					last;
				}
				$score += $mat{$mat_name}{($k-$j+1)}{$sequence_nts[$k]};
			}
			push @{$score_mat{$mat_name}},sprintf("%.3f",$score);
		}
	}
}

