#! /usr/bin/perl -w
# Created by Yiming Cheng on 2006/02/22
# Function: Predict the poly(A) site of the input file containing FASTA format sequences.
# 	    The output format depends on the option [-m mode] c/s, c---complex,s---simple.
#	    If the [-l location] option is given, it will only do one hit model
# Comments: modified on 2006/05/04 by Yiming for simplification.
#ver. 1.1, bug fix on match_seq.pl

use strict;
use Getopt::Std;

my %args;
getopt("ioeml",\%args);

sub exit_help{
	print "\nUsage: perl polya_svm.pl -i input [-o output] [-e cut_off] [-m mode] [-l location]\n";
	print "Options:\n";
	print "	-i input --- file containing the FASTA format sequences. \n";
	print "	-o output --- the output file, the output format is based on the mode.\n";
	print "	-e evalue --- the default evalue is 0.5, the lower the evalue, the more accuracy.\n";
	print "	-m mode --- if 'c', complex format output including the 15 cis elements scores.\n";
	print "		    if 's', simple format output, the default mode.\n";
	print "		    if 'e', only the match score result.\n";
	print "	-l location --- if this is given, it will do one hit model, just predict if the given location is a poly(A) site or not.\n\n";
	exit;
}

my ($input,$output,$cut_off,$mode,$location);
&ini_options();

sub ini_options{
	$input = $args{i};
	if (!defined($input)){
		&exit_help();
	}
	$output = $args{o};
	if (!$output){
		$output = "predict.junk";
	}
	$cut_off = $args{e};
	if (!defined($cut_off)){
		$cut_off =0.5;
	}
	$mode = $args{m};
	if (!defined($mode)){
		$mode = 's';
	}
	$location = $args{l};
}

#my $time1=time();

if (defined($location)){
	system("perl match_seq.pl -i $input -o res.junk -n 1 -l $location");
} else {
	system("perl match_seq.pl -i $input -o res.junk -n 1");
}

if ($mode eq 'e'){
	system("mv $input.score $output");
	if ($output eq "predict.junk"){
		open(JUNK,"<$output");
		while(<JUNK>){
			chomp;
			print $_,"\n";
		}
		close(JUNK);
	}
	system("rm *.junk");
	system("rm $input.num");
	exit;
}

#my $time2=time();

system("perl scale.pl -r ./data/range1 -i res.junk -o dat.junk");
system("./svm-predict -b 1 dat.junk ./data/train1.model out.junk");

#my $time3=time();

system("perl predict_m.pl -i out.junk -c $input.num -o $output -e $cut_off");
if ($mode eq 'c'){
	### Complex output
	my (%all_loc,@all_pre,@all_score);
	open(OUT,"$output");
	my $temp_id;
	while(<OUT>){
		chomp;
		if ($_ =~/^>([\d\w\.]+)/){
			$temp_id = $1;
			$all_loc{$temp_id}='';
		} elsif ($_ =~/^\+/){
			my @line = split(/\t/,$_);
			if ($line[1]=~/M/){
				$all_loc{$temp_id} = 1;
			} else {
				my $junk_junk = $all_loc{$temp_id};
				if ($junk_junk eq ''){
					$all_loc{$temp_id} =($line[3]-100);
				} else {
					$all_loc{$temp_id} = $junk_junk."+".($line[3]-100);
				}
			}
		} elsif ($_ =~/^\-/){
			$all_loc{$temp_id}=0;
		}
	}
	close(OUT);
	open(TEMP_OUT,"<out.junk");
	while(<TEMP_OUT>){
		chomp;
		next if ($.==1);
		my @line = split(/ /,$_);
		push @all_pre,sprintf("%.4f",$line[1]);
	}
	close(TEMP_OUT);
	open(RES,"<res.junk");
	while(<RES>){
		chomp;
		my @line =split(/\t/,$_);
		push @all_score,join("\t",@line[1..$#line]);
	}
	close(RES);
	open(NUM,"<$input.num");
	open(COM,">complex.junk");
	print COM "#+/-\tp-pro\tLocation\tAUE1\tAUE2\tAUE3\tAUE4\tCUE1\tCUE2\tCDE1\tCDE2\tCDE3\tCDE4\tADE1\tADE2\tADE3\tADE4\tADE5\n";
	my $j=0;
	while(<NUM>){
		chomp;
		my @line = split(/\t/,$_);
		print COM ">$line[0]\n";
		if ($line[1]==0){
			print COM "Sequence too short. No prediction\n";
			next;
		} elsif ($line[1] == 1){
			if ($all_pre[$j]>0.5){
				if ($line[2] ==0){
					print COM "+++++","\t",$all_pre[$j],"\tMiddle\t",$all_score[$j],"\n";
				} else {
					print COM "+++++","\t",$all_pre[$j],"\t$line[2]\t",$all_score[$j],"\n";
				}
			} else {
				if ($line[2] ==0){
					print COM "-","\t",$all_pre[$j],"\tMiddle\t",$all_score[$j],"\n";
				} else {
					print COM "-","\t",$all_pre[$j],"\t$line[2]\t",$all_score[$j],"\n";
				}
			}
		} else {
			my $temp_loc = $all_loc{$line[0]};
			my @temp_loc_arr;
			my $k= -1;
			if (($temp_loc ne 0)&&($temp_loc ne 1)){
				@temp_loc_arr = split(/\+/,$temp_loc);
				$k=0;
			}
			if ($k<0){
				for (my $i=0;$i<$line[1];$i++){
					if ($all_pre[$i+$j]>=0.5){
						print COM "+";
					} else {
						print COM "-";
					}
					print COM "\t",$all_pre[$i+$j],"\t",$i+100,"\t",$all_score[$i+$j],"\n";
				}
			} else {
				for (my $i=0;$i<$line[1];$i++){
					if (($k<=$#temp_loc_arr)&&($i==$temp_loc_arr[$k])){
						print COM "+++++\t",$all_pre[$i+$j],"\t",$i+100,"\t",$all_score[$i+$j],"\n";
						$k++;
					} else {
						if ($all_pre[$i+$j]>=0.5){
							print COM "+","\t",$all_pre[$i+$j],"\t",$i+100,"\t",$all_score[$i+$j],"\n";
						} else {
							print COM "-","\t",$all_pre[$i+$j],"\t",$i+100,"\t",$all_score[$i+$j],"\n";
						}
					}
				}
			}
		}	
		$j = $j+ $line[1];
	}
	close(NUM);
	close(COM);
	system("mv complex.junk $output");
}
system("rm $input.score");

#my $time4=time();
#print "Time for match is ",$time2-$time1,"\n";
#print "Time for scale and svm prediction is ",$time3-$time2,"\n";
#print "Time for optimize is ",$time4-$time3,"\n";

if ($output eq "predict.junk"){
	open(JUNK,"<$output");
	while(<JUNK>){
		chomp;
		print $_,"\n";
	}
	close(JUNK);
}
system("rm *.junk");
system("rm $input.num");

